@@ -1390,14 +1390,12 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 				tmp = xmlGetProp (node, (const xmlChar *)"HREF");
 			if (tmp == NULL)
 				continue;
-			if (url == NULL || g_str_has_prefix ((char *)tmp, "mms:") != FALSE) {
-				if (url)
-					xmlFree (url);
+			/* FIXME, should we prefer mms streams, or non-mms?
+			 * See bug #352559 */
+			if (url == NULL)
 				url = tmp;
-			} else {
-				if (tmp)
-					xmlFree (tmp);
-			}
+			else
+				xmlFree (tmp);
 
 			continue;
 		}
@@ -1553,7 +1551,7 @@ parse_smil_video_entry (TotemPlParser *parser, char *base,
 
 static gboolean
 parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
-		xmlNodePtr parent)
+		xmlNodePtr parent, xmlChar *parent_title)
 {
 	xmlNodePtr node;
 	xmlChar *title, *url;
@@ -1574,7 +1572,9 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 			if (url != NULL) {
 				if (parse_smil_video_entry (parser,
-						base, (char *)url, (char *)title) != FALSE)
+						base, (char *)url,
+						title ? (char *)title
+						: (char *)parent_title) != FALSE)
 					retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 			}
 
@@ -1584,7 +1584,7 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 				xmlFree (url);
 		} else {
 			if (parse_smil_entry (parser,
-						base, doc, node) != FALSE)
+						base, doc, node, parent_title) != FALSE)
 				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 		}
 	}
@@ -1592,12 +1592,37 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 	return retval;
 }
 
+static xmlChar *
+parse_smil_head (TotemPlParser *parser, xmlDocPtr doc, xmlNodePtr parent)
+{
+	xmlNodePtr node;
+	xmlChar *title = NULL;
+
+	for (node = parent->children; node != NULL; node = node->next) {
+		if (g_ascii_strcasecmp ((char *)node->name, "meta") == 0) {
+			xmlChar *prop;
+			prop = xmlGetProp (node, (const xmlChar *)"name");
+			if (prop != NULL && g_ascii_strcasecmp ((char *)prop, "title") == 0) {
+				title = xmlGetProp (node, (const xmlChar *)"content");
+				if (title != NULL) {
+					xmlFree (prop);
+					break;
+				}
+			}
+			xmlFree (prop);
+		}
+	}
+
+	return title;
+}
+
 static gboolean
 parse_smil_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
+	xmlChar *title = NULL;
 
 	for (node = parent->children; node != NULL; node = node->next) {
 		if (node->name == NULL)
@@ -1605,12 +1630,17 @@ parse_smil_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 		if (g_ascii_strcasecmp ((char *)node->name, "body") == 0) {
 			if (parse_smil_entry (parser, base,
-						doc, node) != FALSE)
+						doc, node, title) != FALSE)
 				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
+		} else if (title == NULL) {
+			if (g_ascii_strcasecmp ((char *)node->name, "head") == 0)
+				title = parse_smil_head (parser, doc, node);
 		}
-
 	}
 
+	if (title != NULL)
+		xmlFree (title);
+
 	return retval;
 }
 
@@ -2048,7 +2078,7 @@ totem_pl_parser_is_asf (const char *data, gsize len)
 		g_warning ("Couldn't dup data in totem_pl_parser_is_asf");
 		return FALSE;
 	}
-	buffer[len] = '\0';
+	buffer[len - 1] = '\0';
 	if (strstr (buffer, "<ASX") != NULL
 			|| strstr (buffer, "<asx") != NULL) {
 		g_free (buffer);
@@ -2119,6 +2149,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("application/vnd.rn-realmedia", totem_pl_parser_add_ra, totem_pl_parser_is_ra),
 	PLAYLIST_TYPE2 ("audio/x-pn-realaudio-plugin", totem_pl_parser_add_ra, totem_pl_parser_is_ra),
 	PLAYLIST_TYPE2 ("audio/vnd.rn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_ra),
+	PLAYLIST_TYPE2 ("audio/x-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_ra),
 	PLAYLIST_TYPE2 ("text/plain", totem_pl_parser_add_ra, totem_pl_parser_is_ra),
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
@@ -2358,6 +2389,8 @@ totem_pl_parser_add_ignored_mimetype (TotemPlParser *parser,
 
 #endif /* !TOTEM_PL_PARSER_MINI */
 
+#define D(x) if (debug) x
+
 gboolean
 totem_pl_parser_can_parse_from_data (const char *data,
 				     gsize len,
@@ -2368,8 +2401,6 @@ totem_pl_parser_can_parse_from_data (const char *data,
 
 	g_return_val_if_fail (data != NULL, FALSE);
 
-#define D(x) if (debug) x
-
 	/* Bad cast! */
 	mimetype = gnome_vfs_get_mime_type_for_data ((gpointer) data, (int) len);
 
@@ -2396,3 +2427,25 @@ totem_pl_parser_can_parse_from_data (const char *data,
 
 	return FALSE;
 }
+
+gboolean
+totem_pl_parser_can_parse_from_filename (const char *filename, gboolean debug)
+{
+	char *contents;
+	gsize len;
+	gboolean retval;
+
+	g_return_val_if_fail (filename != NULL, FALSE);
+
+	/* FIXME probably a good idea to only get the first few chunks */
+	if (g_file_get_contents (filename, &contents, &len, NULL) == FALSE) {
+		D(g_message ("couldn't get the contents of %s", filename));
+		return FALSE;
+	}
+
+	retval = totem_pl_parser_can_parse_from_data (contents, len, debug);
+	g_free (contents);
+
+	return retval;
+}
+
